/* $Novell: ssl_any.c,v 1.10 2003/05/12 13:08:29 $ */
/**************************************************************************
*  Novell Software Developer Kit
*
*  Copyright (C) 2002-2003 Novell, Inc. All Rights Reserved.
*
*  THIS WORK IS SUBJECT TO U.S. AND INTERNATIONAL COPYRIGHT LAWS AND TREATIES.
*  USE AND REDISTRIBUTION OF THIS WORK IS SUBJECT TO THE LICENSE AGREEMENT
*  ACCOMPANYING THE SOFTWARE DEVELOPER KIT (SDK) THAT CONTAINS THIS WORK.
*  PURSUANT TO THE SDK LICENSE AGREEMENT, NOVELL HEREBY GRANTS TO DEVELOPER A
*  ROYALTY-FREE, NON-EXCLUSIVE LICENSE TO INCLUDE NOVELL'S SAMPLE CODE IN ITS
*  PRODUCT. NOVELL GRANTS DEVELOPER WORLDWIDE DISTRIBUTION RIGHTS TO MARKET,
*  DISTRIBUTE, OR SELL NOVELL'S SAMPLE CODE AS A COMPONENT OF DEVELOPER'S
*  PRODUCTS. NOVELL SHALL HAVE NO OBLIGATIONS TO DEVELOPER OR DEVELOPER'S
*  CUSTOMERS WITH RESPECT TO THIS CODE.
*
***************************************************************************
    ssl_any.c
***************************************************************************
   Description: Demonstrates making an ssl connection to an ldap server
      without any server certification verification.
***************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include "ldap_ssl.h"
    
#if defined(N_PLAT_NLM) && defined(LIBC)
#include <screen.h>
#endif
    
static char usage[] =
"\n Usage:   ssl_any <host name> <port number> <login dn> <password>\n"
"  host name      = ldap server name or IP address\n"
"  port number    = port to use - 636 is ldap ssl default\n" 
"  login dn       = user name to login as\n"
"  password       = user password\n"
"\n Example: ssl_any Acme.com 636 cn=admin,o=Acme secret\n";

int main(int argc, char *argv[])
{

    struct timeval timeOut = {10,0};   /* 10 second connection timeout */
    int  version, rc, ldapPort;
    char *ldapHost;
    char *loginDN;
    char *password;
    char *attrs[]   = { LDAP_NO_ATTRS, NULL };

    LDAP           *ld;
    LDAPMessage    *resultBuf = NULL;
    LDAPMessage    *entry = NULL;
    char           *dn;
       
    #if defined(N_PLAT_NLM) && defined(LIBC)
    setscreenmode(SCR_NO_MODE);              /* Don't clear screen on exit */
    #endif
    
    if (argc != 5)
    {
        printf("%s", usage);
        exit(1);
    }

    ldapHost     = argv[1];
    ldapPort     = atoi(argv[2]);
    loginDN      = argv[3];
    password     = argv[4];
   

    /* Set LDAP version to 3 and set connection timeout. */
    version = LDAP_VERSION3;
    ldap_set_option( NULL, LDAP_OPT_PROTOCOL_VERSION, &version);
    ldap_set_option( NULL, LDAP_OPT_NETWORK_TIMEOUT, &timeOut);

    /* 
     * Initialize the ssl library.  The first parameter of 
     * ldapssl_client_init is a certificate file.  However, when used
     * the file must be a DER encoded file.  NULL is passed in for the
     * certificate file because ldapssl_set_verify_mode will be used
     * to specify no server certificate verification. 
     *
     * ldapssl_client_init is an application level initialization not a 
     * thread level initilization and should be done once.
     */
    rc = ldapssl_client_init(   NULL,       /* DER encoded cert file  */ 
                                NULL );     /* reserved, use NULL */

    if (rc != LDAP_SUCCESS)
    {
        printf("ldapssl_client_init error: %d\n", rc);
        exit(1);
    }


    /*
     * Configure the LDAP SSL library to not verify the server certificate.  
     * The default is LDAPSSL_VERIFY_SERVER which validates all servers 
     * against the trusted certificates normally passed in 
     * during ldapssl_client_init or ldapssl_add_trusted_cert.
     *
     * WARNING:  Setting the verify mode to LDAPSSL_VERIFY_NONE turns off
     * server certificate verification.  This means all servers are 
     * considered trusted.  This should be used only in controlled 
     * environments where encrypted communication between servers and 
     * clients is desired but server verification is not necessary.
     */   
    rc = ldapssl_set_verify_mode(LDAPSSL_VERIFY_NONE);
    if (rc != LDAP_SUCCESS)
    {
        printf("ldapssl_set_verify_mode error: %d\n", rc);
        ldapssl_client_deinit();
        exit(1);
    }


    /* 
     * create a LDAP session handle that is enabled for ssl connection
     */
    ld = ldapssl_init(  ldapHost,       /* host name */
                        ldapPort,       /* port number */
                        1 );            /* 0- clear text, 1-enable for ssl */

    if (ld == NULL )
    {
        printf("ldapssl_init error\n" );
        ldapssl_client_deinit();
        exit(1);
    }

    rc = ldap_simple_bind_s( ld, loginDN, password);
   
    if (rc != LDAP_SUCCESS )
    {
        printf("ldap_simple_bind_s error: %d, %s\n", rc, ldap_err2string( rc ));
        ldap_unbind_s( ld );
        ldapssl_client_deinit();
        exit(1);
    }
    

    printf("SSL bind successful - performing search\n");
      

    /* 
     * Do a simple search just to verify things worked
     */
    rc = ldap_search_ext_s(ld, "" , LDAP_SCOPE_ONELEVEL, "(objectClass=*)",
                           attrs, 0, NULL, NULL, NULL, 
                           LDAP_NO_LIMIT, &resultBuf);

    if( rc == LDAP_SUCCESS)
    {
       entry = ldap_first_entry(ld, resultBuf);

       while (entry != NULL)
       {
          dn = ldap_get_dn(ld, entry);
          printf("\tObject: %s\n", dn);

          ldap_memfree(dn);

          entry = ldap_next_entry(ld, entry);
       }

       ldap_msgfree( resultBuf );
    }
    else
    {
       printf("ldap_search_s error: %d, %s\n", rc, ldap_err2string(rc));
    }

    ldap_unbind_s( ld );

    /* 
     * Uninitialize the LDAP ssl library
     */
    ldapssl_client_deinit();

    return 0;
}

